# Generated by ariadne-codegen
# Source: vals/graphql/

from datetime import datetime
from typing import Any

from pydantic import Field

from .base_model import BaseModel


class GetTestSuiteData(BaseModel):
    test_suite: "GetTestSuiteDataTestSuite" = Field(alias="testSuite")


class GetTestSuiteDataTestSuite(BaseModel):
    description: str
    id: str
    org: str
    title: str
    created: datetime
    global_checks: Any = Field(alias="globalChecks")


GetTestSuiteData.model_rebuild()
