# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Any, List, Optional

from pydantic import Field

from .base_model import BaseModel


class AddBatchTests(BaseModel):
    batch_update_test: Optional["AddBatchTestsBatchUpdateTest"] = Field(
        alias="batchUpdateTest"
    )


class AddBatchTestsBatchUpdateTest(BaseModel):
    tests: List["AddBatchTestsBatchUpdateTestTests"]


class AddBatchTestsBatchUpdateTestTests(BaseModel):
    checks: Any
    test_id: str = Field(alias="testId")
    cross_version_id: str = Field(alias="crossVersionId")
    file_ids: Any = Field(alias="fileIds")
    input_under_test: str = Field(alias="inputUnderTest")
    tags: Any
    context: Any
    golden_output: str = Field(alias="goldenOutput")
    test_suite: "AddBatchTestsBatchUpdateTestTestsTestSuite" = Field(alias="testSuite")


class AddBatchTestsBatchUpdateTestTestsTestSuite(BaseModel):
    id: str


AddBatchTests.model_rebuild()
AddBatchTestsBatchUpdateTest.model_rebuild()
AddBatchTestsBatchUpdateTestTests.model_rebuild()
