# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Optional

from pydantic import Field

from .base_model import BaseModel


class CreateOrUpdateTestSuite(BaseModel):
    update_test_suite: Optional["CreateOrUpdateTestSuiteUpdateTestSuite"] = Field(
        alias="updateTestSuite"
    )


class CreateOrUpdateTestSuiteUpdateTestSuite(BaseModel):
    test_suite: "CreateOrUpdateTestSuiteUpdateTestSuiteTestSuite" = Field(
        alias="testSuite"
    )


class CreateOrUpdateTestSuiteUpdateTestSuiteTestSuite(BaseModel):
    description: str
    id: str
    org: str
    title: str


CreateOrUpdateTestSuite.model_rebuild()
CreateOrUpdateTestSuiteUpdateTestSuite.model_rebuild()
