# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Optional

from pydantic import Field

from .base_model import BaseModel


class CreateRagSuite(BaseModel):
    update_rag_suite: Optional["CreateRagSuiteUpdateRagSuite"] = Field(
        alias="updateRagSuite"
    )


class CreateRagSuiteUpdateRagSuite(BaseModel):
    rag_suite: Optional["CreateRagSuiteUpdateRagSuiteRagSuite"] = Field(
        alias="ragSuite"
    )


class CreateRagSuiteUpdateRagSuiteRagSuite(BaseModel):
    id: str
    query: str


CreateRagSuite.model_rebuild()
CreateRagSuiteUpdateRagSuite.model_rebuild()
