# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import List

from pydantic import Field

from .base_model import BaseModel


class GetRagSuites(BaseModel):
    rag_suites: List["GetRagSuitesRagSuites"] = Field(alias="ragSuites")


class GetRagSuitesRagSuites(BaseModel):
    id: str
    org: str
    path: str
    query: str


GetRagSuites.model_rebuild()
